@@ -872,3 +872,232 @@ impl ContainerCreateOptsBuilder {
         work_dir => "work_dir"
     );
 }
+
+impl_opts_builder!(url =>
+    /// Adjust how to attach to a running container.
+    ContainerAttach
+);
+
+impl ContainerAttachOpts {
+    pub(crate) fn stream(&self) -> Self {
+        let mut new = self.clone();
+        new.params.insert("stream", true.to_string());
+        new
+    }
+}
+
+impl ContainerAttachOptsBuilder {
+    impl_url_str_field!(
+        /// Keys to use for detaching from the container.
+        detach_keys => "detachKeys"
+    );
+
+    impl_url_bool_field!(
+        /// Attach to container STDERR
+        stderr => "stderr"
+    );
+
+    impl_url_bool_field!(
+        /// Attach to container STDIN
+        stdin => "stdin"
+    );
+
+    impl_url_bool_field!(
+        /// Attach to container STDOUT
+        stdout => "stdout"
+    );
+}
+
+impl_opts_builder!(url =>
+    /// Adjust how to attach to a running container.
+    ContainerLogs
+);
+
+impl ContainerLogsOptsBuilder {
+    impl_url_bool_field!(
+        /// Keep connection after returning logs.
+        follow => "follow"
+    );
+
+    impl_url_str_field!(
+        /// Only return logs since this time, as a UNIX timestamp.
+        since => "since"
+    );
+
+    impl_url_bool_field!(
+        /// Return logs from STDERR.
+        stderr => "stderr"
+    );
+
+    impl_url_bool_field!(
+        /// Return logs from STDOUT.
+        stdout => "stdout"
+    );
+
+    impl_url_str_field!(
+        /// Only return this number of log lines from the end of the logs. Default - `all`
+        tail => "tail"
+    );
+
+    impl_url_bool_field!(
+        /// Add timestamps to every log line.
+        timestamps => "timestamps"
+    );
+
+    impl_url_str_field!(
+        /// Only return logs before this time, as a UNIX timestamp.
+        until => "until"
+    );
+}
+
+impl_opts_builder!(url =>
+    /// Adjust how container stats are reported.
+    ContainerStats
+);
+
+impl ContainerStatsOpts {
+    pub(crate) fn oneshot(&self) -> Self {
+        let mut new = self.clone();
+        new.params.insert("stream", false.to_string());
+        new
+    }
+
+    pub(crate) fn stream(&self) -> Self {
+        let mut new = self.clone();
+        new.params.insert("stream", true.to_string());
+        new
+    }
+}
+
+impl ContainerStatsOptsBuilder {
+    impl_url_vec_field!(
+        /// Names or IDs of containers
+        containers => "containers"
+    );
+
+    impl_url_field!(
+        /// Time in seconds between stats reports
+        interval: usize => "interval"
+    );
+}
+
+impl_opts_builder!(url =>
+    /// Adjust how container stats are reported.
+    ContainerTop
+);
+
+impl ContainerTopOpts {
+    pub(crate) fn oneshot(&self) -> Self {
+        let mut new = self.clone();
+        new.params.insert("stream", false.to_string());
+        new
+    }
+
+    pub(crate) fn stream(&self) -> Self {
+        let mut new = self.clone();
+        new.params.insert("stream", true.to_string());
+        new
+    }
+}
+
+impl ContainerTopOptsBuilder {
+    impl_url_field!(
+        /// if streaming, delay in seconds between updates. Must be >1. (As of version 4.0)
+        delay: usize => "delay"
+    );
+
+    impl_url_str_field!(
+        /// Arguments to pass to ps such as aux. Requires ps(1) to be installed in the container
+        /// if no ps(1) compatible AIX descriptors are used.
+        ps_args => "ps_args"
+    );
+}
+
+#[derive(Debug)]
+/// Used to filter removed images.
+pub enum ContainerPruneFilter {
+    /// Prune containers created before this timestamp. The <timestamp> can be Unix timestamps, date
+    /// formatted timestamps, or Go duration strings (e.g. 10m, 1h30m) computed relative to the
+    /// daemon machine’s time.
+    // #TODO: DateTime
+    Until(String),
+    /// Container that contains key label.
+    LabelKey(String),
+    /// Container that contains key-value label.
+    LabelKeyVal(String, String),
+}
+
+impl Filter for ContainerPruneFilter {
+    fn query_key_val(&self) -> (&'static str, String) {
+        use ContainerPruneFilter::*;
+        match &self {
+            Until(until) => ("until", until.to_string()),
+            LabelKey(key) => ("label", key.clone()),
+            LabelKeyVal(key, val) => ("label", format!("{}={}", key, val)),
+        }
+    }
+}
+
+impl_opts_builder!(url =>
+    /// Adjust how stopped containers are removed.
+    ContainerPrune
+);
+
+impl ContainerPruneOptsBuilder {
+    impl_filter_func!(
+        /// Filters to process on the prune list.
+        ContainerPruneFilter
+    );
+}
+
+impl_opts_builder!(url =>
+    /// Adjust how a container is restored.
+    ContainerRestore
+);
+
+impl ContainerRestoreOptsBuilder {
+    impl_url_bool_field!(
+        /// Do not include root file-system changes when exporting.
+        ignore_root_fs => "ignoreRootFS"
+    );
+
+    impl_url_bool_field!(
+        /// Ignore IP address if set statically.
+        ignore_static_ip => "ignoreStaticIP"
+    );
+
+    impl_url_bool_field!(
+        /// Ignore MAC address if set statically.
+        ignore_static_mac => "ignoreStaticMac"
+    );
+
+    impl_url_bool_field!(
+        /// Import the restore from a checkpoint tar.gz.
+        import => "import"
+    );
+
+    impl_url_bool_field!(
+        /// Keep all temporary checkpoint files.
+        keep => "keep"
+    );
+
+    impl_url_bool_field!(
+        /// Leave the container running after writing checkpoint to disk.
+        leave_running => "leaveRunning"
+    );
+
+    impl_url_str_field!(
+        /// The name of the container when restored from a tar. can only be used with import.
+        name => "name"
+    );
+
+    impl_url_bool_field!(
+        /// Add restore statistics to the response.
+        print_stats => "printStats"
+    );
+
+    impl_url_bool_field!(
+        /// Checkpoint a container with established TCP connections.
+        tcp_established => "tcpEstablished"
+    );
+}
